from typing import Any
from django.contrib.auth.models import AbstractUser
from django.db import models
from ldap_inp import LdapEleve, LdapClub
from ldap_inp import LdapEleve, LdapClub


USER_TYPE = [
    ("ETU", "ETUDIANT"),
    ("ADM", "ADMINISTRATION"),
    ("RC", "RESPONSABLE_CLUB"),
    ("STI", "STI"),
]

REQUEST_STATE = [
    ("REF", "REFUSE"),
    ("BR", "BROUILLON"),
    ("ATT_VAL", "EN_ATTENTE_VALIDATION"),
    ("VAL_RC", "VALIDE_RESPONSABLE_CLUB"),
    ("VAL_ADM", "VALIDE_ADMIN"),
    ("VAL_STI", "VALIDE_STI"),
    ("MOD", "MODIFICATION_DEMANDEE"),
]


class User(AbstractUser):
    username = models.CharField(max_length=100, unique=True)
    password = models.CharField(max_length=100, null=True)
    # Type de l'utilisateur (ETU, ADM, RC, STI)
    type = models.fields.CharField(choices=USER_TYPE, max_length=10)
    USERNAME_FIELD = "username"

    def __str__(self):
        return self.username

    def get_type(self):
        return self.type

    # Renvoie les club où l'utilisateur est au bureau
    # TODO remplacer 'valliet' par my_uid en prod
    def get_board_clubs(self):
        my_uid = self.username
        my_clubs = LdapEleve.objects.get(uid="valliet").get_clubs()
        ret = []
        for club in my_clubs:
            if "valliet" in [
                item
                for l in [
                    [club.president],
                    club.vicePresident,
                    club.tresorier,
                    club.secretaire,
                ]
                for item in l
            ]:
                ret.append((club.cn, club.displayName))
        return ret

    def is_board(self, club_cn):
        try:
            club = LdapClub.objects.get(cn=club_cn)
            if self.username in [
                item
                for l in [
                    [club.president],
                    club.vicePresident,
                    club.tresorier,
                    club.secretaire,
                ]
                for item in l
            ]:
                return True
            else:
                return False
        except:
            return False

    def is_respoClub(self):
        # TODO
        return False


class Zone(models.Model):
    name = models.CharField(max_length=100)
    description = models.CharField(max_length=1000)

    def __str__(self):
        return self.name


class Event(models.Model):
    name = models.CharField(max_length=100)
    start = models.DateField()
    end = models.DateField()
    location = models.CharField(max_length=100)
    # Mettre la possibilité de choisir de mettre un char pour nbr de participants
    nb_participants = models.IntegerField()
    # Présence d'alcool
    alcohol = models.BooleanField()
    alcohol_details = models.CharField(max_length=100, null=True, blank=True)
    description = models.CharField(max_length=1000)
    admin_comment = models.CharField(max_length=1000, null=True, blank=True)
    creator = models.ForeignKey(User, on_delete=models.PROTECT, null=True, blank=True)

    def __str__(self):
        return self.name


class Request(models.Model):
    event = models.ForeignKey(Event, on_delete=models.CASCADE)
    # Club à l'origine de la demande (Lié à un club du ldap)
    club = models.CharField(max_length=100)
    start = models.DateTimeField()
    end = models.DateTimeField()
    contact_phone = models.CharField(max_length=256)
    contact_mail = models.CharField(max_length=100, default="default@mail.com")
    zones = models.ManyToManyField(Zone)
    # Liste des username ldap des étudiant concernés par la demande (séparés par des virgules)
    authorized = models.CharField(max_length=5000)
    # Commentaire du créateur
    creator_comment = models.CharField(max_length=1000, null=True, blank=True)
    admin_comment = models.CharField(max_length=1000)
    creator = models.ForeignKey(User, on_delete=models.PROTECT)
    # Commentaire de l'admin
    # Etat de la requête (0 = refusé, 1 = en attente de validation, 2 = validé respoclub,
    # 3 = validé admin, 4 = validé sti, 5 = modification demandée par l'admin)
    state = models.fields.CharField(choices=REQUEST_STATE, max_length=20)


class Alarm(models.Model):
    name = models.CharField(max_length=100)
    description = models.CharField(max_length=1000)
    # Emplacement de l'alarme
    location = models.CharField(max_length=100)
    # Zone protégée par l'alarme
    zone = models.ForeignKey(Zone, on_delete=models.CASCADE)
